% The script is developed by Kathrine Gregersen for the master thesis.

% This script calculates the individual- and weighted histograms for
% bending moment timeseries and calculates the fatigue damage.

% This script uses RFC including residuals. The half-cycles is given half
% number of occurence, when added together with the full cycles.

% To run this script the functions readcsc and count_unique are needed, in 
% addition to the WAFO package. From analysis a input file with the bending 
% moment time series and a textfile with the scatter.

%% Clear
clc
clear
close all


%% Read in results from SIMA in a separate function
[moment]=readcsv();
[o,n] = size(moment);
timestep=0.1;
tend=o*timestep;
Time=0.1:timestep:tend;
Time=Time';


%% ----------- Development of histograms and weighted-histogram -------- %%
n_overpull=9;           % Number of overpulls analysed     
n_tp=21;                % Number of peak periods analysed

% SN-curve parameters: DNVGL-RP-0005: Class F1, seawater w/ cathodic protection
m=5;
log_a=14.832;
a=10^log_a;


%plotstyle
plotStyle = {'b','k','r','g','c','m','y',[1 0.270588 0],[0.180392 0.545098 0.341176]};


for k=1:n_overpull      % Loop for each overpull
x=0; z=0; hist_weighed=[]; unique_bending=[];  % clearing variables to get ready for next round
D=[]; D_weighted=[]; 

for i=1:n_tp            % Loop for each Tp
hist_unweighed=[]; uniques_sorted=[];            % clearing variables to get ready for next round

moment(:,i*k)=moment(:,i*k)-mean(moment(:,i*k)); % Removing the mean value, due to current
xx_moment=[Time, moment(:,i*k)];

%% Extraction of turning points
[tp_moment ind]=dat2tp(moment(:,i*k));      % ind identifies in which place in the matrix the turning points occurs.  

% Visualisation of bending moment time series including turning points
% figure(1) 
% ind1=ind/10;            % adjusting it to the timestep which is 0.1 sec
% plot(Time , xx_moment(:,2))
% hold on
% plot(ind1,xx_moment(ind,2),'s','markerfacecolor',[0,0,0], 'markeredgecolor',[0,0,0])
%     title(['Time series w/ turning points, Overpull ', num2str(k),', Tp=' , num2str(i+3.5)])
% xlabel('Time [s]')
% ylabel('Bending Moment [Nm]')
% %xlim([0 tend])
% xlim([0 500])
% hold off


%% ----------------------- Rainflow Counting ------------------------ %%
[RFC,RFC_WOR,R]=tp2rfc(tp_moment);    % RFC=RFC w/residual, RFC_WOR=RFC w/o residual, R=residual.

%% ------------ Calculate moment ranges and number of cycles -------- %%
% Moment range for whole cycles
RFC_WOR(:,3)=abs(RFC_WOR(:,2)-RFC_WOR(:,1));

% Moment range for half cycles
n_res=numel(R); %counts the number of residuals
for j = 2:2:n_res
R(j,3) = abs(R(j,1)-R(j-1,1));
end

% Round up every moment range to establish histogram. Each bin is 5000Nm.
RFC_WOR(:,4)=ceil(RFC_WOR(:,3)/5000)*5000;
R(:,4)=ceil(R(:,3)/5000)*5000;


% Find and count number of unique moment ranges
% For whole cycles:
[uniques_WOR,numUnique_WOR] = count_unique(RFC_WOR(:,4));  
% count_unique is a function made by Anthony Kendall.

% For half cycles:
[uniques_R,numUnique_R] = count_unique(round(R(:,4)));

% To ensure that the unique count will represent the number of full cycles
% the uniquecount of half cycles is divided by two: 
numUnique_R = numUnique_R./2;

% Add the whole- and half-cycles together and sorting them
uniques=[uniques_WOR; uniques_R];
numUnique=[numUnique_WOR; numUnique_R];

uniques_sorted = unique(uniques);        % MATLAB function that returns data without repetitions
for j = 1:length(uniques_sorted)
hist_unweighed(j,1) = uniques_sorted(j,1);
hist_unweighed(j,2) = sum(numUnique(uniques == uniques_sorted(j,1)));
end

if hist_unweighed(1,1) == 0             % Removing zeros if occuring
hist_unweighed(1,:) = [];
end

% Histogram plot for every Overpull & Tp Combination
if k == 7
figure(1+i*k)
h=bar(hist_unweighed(:,1),hist_unweighed(:,2),'histc');
sh=findall(gcf,'marker','*');
delete(sh);
set(h,'FaceColor','b','EdgeColor','k');
xlim([0,800000])
ylim([0,80])
title(['Histogram for Hs=3.5, Overpull= ', num2str(k), ', Tp= ', num2str(i+3.5)])
ylabel('Number of cycles')
xlabel('Moment range [Nm]')
fprintf('Total number of whole cycles is: %5.2f\n',...
sum(numUnique_WOR))
fprintf('Total number of half cycles is: %5.2f\n',...
sum(numUnique_R))
fprintf('Total number of cycle bins is: %5.2f\n\n',...
length(hist_unweighed(:,1)))
end 


%% ---------- Weighted moment ranges and number of cycles ---------  %%
% Read in the no.of occurences for each tp
load scatter2.txt    % txt-file with the scattter weight for each Tp
S=sum(scatter2);     % total number of occurences


% Collecting all unique cycles into same vector
y = [hist_unweighed(:,1)];
% Collecting all unique number count for whole cycles
% into same vector and weighing by the probability
% of occurence for current sea state
w = [hist_unweighed(:,2)]*scatter2(i)/S;
% Adding to previous collected whole cycles
% and whole cycle counts
z = [z; y]; x = [x; w];
clear b1 a1         % clearing variables to get ready for next round

%% --------------------- Damage Calculations ----------------------- %%
%fullcycles
D_full=(1/a).*sum(numUnique_WOR.*(uniques_WOR.^m)); %summing all blocks for one Tp
%half-cycles
D_half=(1/a).*sum(numUnique_R.*(uniques_R.^m));
% adding them together
D(:,i)=D_full+D_half;
% Weighted damage
D_weighted(:,i)=D(:,i).*scatter2(i)/S;


end                 % End loop for i=1:21 (Tp-loop)


% Collecting all whole cycles and adding all weighted cycle count
% contribution from each Tp.
if z(1) == 0                       % Remove zeros if occuring
x(1) = []; z(1) = [];             
end

unique_bending = unique(z);        % MATLAB function that returns data without repetiotions
for j = 1:length(unique_bending)
hist_weighed(j,1) = unique_bending(j,1);
hist_weighed(j,2) = sum(x(z == unique_bending(j,1)));
end

%% Weighted histogram for Hs=3.5 all Tp, overpull k
figure(n+1+k);
h=bar(hist_weighed(:,1),hist_weighed(:,2),'histc')
sh=findall(gcf,'marker','*');
delete(sh);
set(h,'FaceColor','b','EdgeColor','k');

xlim([0,2100000])
ylim([0,50])
ylabel('Number of cycles')
xlabel('Bending Moment Range [Nm]')
title(['Weighted histogram: 125m, all Tp, Overpull ', num2str(k)])         %change name depending on waterdepth
fprintf('Total number of weighted cycles for all Hs and all Tp is: %5.2f\n',...
sum(hist_weighed(:,2)))
fprintf('Total number of weighted cycle bins is: %5.2f\n\n',...
length(hist_weighed(:,1)))

%% Adding damage for all Tp into one matrix
D_alltp(k,:)=D;
D_weighted_alltp(k,:)=D_weighted;

% %plot figure
hFig=figure (1001);
%set(hFig, 'Position', [0 0 2000 1000])
%bar(hist_weighed(:,1),hist_weighed(:,2),'FaceColor',plotStyle{k},'EdgeColor',plotStyle{k});
if k==1
[p1,g1] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
elseif k==2
[p2,g2] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
elseif k==3
[p3,g3] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
elseif k==4
[p4,g4] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
elseif k==5
[p5,g5] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
elseif k==6
[p6,g6] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
elseif k==7
[p7,g7] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
elseif k==8
[p8,g8] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
else
[p9,g9] = createPatches(hist_weighed(:,1)/1000,hist_weighed(:,2),2.5,plotStyle{k},0.5);
end

title('Bending Moment Histogram: Overpull Variation - 80m');
xlabel('Bending moment range [kNm]');
ylabel('Number of cycles');

hold on

% %plot figure
% figure (1002)
% plot(hist_weighed(:,1),hist_weighed(:,2));%, plotStyle{k});
% legendInfo{k} = ['Overpull' num2str(k)];
% hold on

%write damage to file
%filename = ['damage/1000m_50red_overpull' num2str(k) '.csv']; 
%csvwrite(filename, hist_weighed);


end             % End loop for k=1:9 (Overpull-loop)

% setting legend for the combined overpull plot
%Use the hggroup for creating the legend and get second output:
[leg,legIcon] = legend([g1 g2 g3 g4 g5 g6 g7 g8 g9],'Overpull 25 t','Overpull 35 t','Overpull 45 t', 'Overpull 55 t', 'Overpull 62.5 t', 'Overpull 70 t', 'Overpull 80 t','Overpull 90 t','Overpull 100 t'); 
%legIcon contains the patch objects of the legend
legPatch = findobj(legIcon,'type','patch');
%Set the FaceAlpha for the legend's patches:
legPatch(1).FaceAlpha = p1(1).FaceAlpha;
legPatch(2).FaceAlpha = p2(2).FaceAlpha;
legPatch(3).FaceAlpha = p3(3).FaceAlpha;
legPatch(4).FaceAlpha = p4(4).FaceAlpha;
legPatch(5).FaceAlpha = p5(5).FaceAlpha;
legPatch(6).FaceAlpha = p6(6).FaceAlpha;
legPatch(7).FaceAlpha = p7(7).FaceAlpha;
legPatch(8).FaceAlpha = p8(8).FaceAlpha;
legPatch(9).FaceAlpha = p9(9).FaceAlpha;

% print to file
%print(['histogram_plots/basecase_190m_overpullvar'],'-djpeg');

%% Sum of weighted damage
Damage_all=sum(D_weighted_alltp,2);             %summing up the contribution from all Tp's
disp('Weighted damage for each overpull is: ')
disp(Damage_all)


